// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <vector>
#include <cstdio>
#include <ctime>

#include "pingubmpmap.hh"
#include "background.hh"
#include "world.hh"
#include "traps/traps.hh"
#include "result.hh"

//CL_Surface* World::gfx_bg = 0;
CL_ResourceManager* World::resource;

World::World()
{
  released_pingus = 0;
}

World::~World()
{
  delete map;
  delete  bg;
}

void 
World::draw_pingus()
{
  for(int i=0; i < pingu.size(); ++i) {
    pingu[i]->draw_offset(x_offset, y_offset);

    map->mark_dirty(pingu[i]->get_x()-16, pingu[i]->get_y()-32,32,32);
    bg->mark_dirty(pingu[i]->get_x()-16,  pingu[i]->get_y()-32,32,32);
  }  
}

// draw the basic background pattern
void 
World::draw_background()
{
  bg_is_drawn = true;
  bg->draw_offset(x_offset, y_offset);

  for(int i=0; i < hotspot_bg.size(); ++i) 
    hotspot_bg[i]->draw_offset(x_offset, y_offset);
  
  for(int i=0; i < traps.size(); ++i) 
    traps[i]->draw_offset(x_offset, y_offset);
}

void 
World::draw_playground()
{
  // Draw the exits
  for(int i=0; i < exits.size(); ++i) {
    exits[i]->draw_offset(x_offset, y_offset);
  }
  
  // Draw the gfx map
  pg_is_drawn = true;
  if (map) map->draw_offset(x_offset, y_offset);

  // Draw the entrances
  for(int i=0; i < entrance.size(); ++i) {
    entrance[i]->draw_offset(x_offset, y_offset);
  }

  // Draw the hotspots
}


void 
World::draw_foreground()
{
  fg_is_drawn = true;
  
  for(int i=0; i < hotspot_fg.size(); ++i) {
    hotspot_fg[i]->draw_offset(x_offset, y_offset);
  }
}

// Merge the different layers on the screen together
void 
World::draw_offset(int x, int y)
{
  x_offset = x;
  y_offset = y;
  
  draw_background();
  draw_playground();
  draw_pingus();
  draw_foreground();
}

void 
World::let_move()
{
  // Create new pingus, if enough time is passed
  for(int i=0; i < entrance.size(); ++i) {
    if (pingu.size() <= allowed_pingus
	&& entrance[i]->pingu_ready()) 
      pingu.push_back(entrance[i]->get_pingu());
    
  }
  
  // Let all pingus move
  for(int i=0; i < pingu.size(); ++i) {
    pingu[i]->let_move();
  }

  // Let the pingus catch each other
  for(int i=0; i < pingu.size(); ++i) {
    for(int j=0; j < pingu.size(); ++j)
      if (pingu[i]->need_catch()) {
	pingu[i]->catch_pingu(pingu[j]);
      }
  }

  // Let the traps catch the pingus
  // Traps are not implemented

  // Let the exit catch the pingus
  for(int j=0; j < exits.size(); ++j) {
    for(int i=0; i < pingu.size(); ++i) {
      exits[j]->catch_pingu(pingu[i]);
    }
  }
}

void 
World::save()
{
  // Empty should save the state of the wolrd 
}

void 
World::load()
{
  // Empty should load the state of the world
}

void
World::load_data(CL_ResourceManager* res)
{
  resource = res;
}

void 
World::init(PLF* level_data, Result& r)
{
  result = &r;

  cout << "Wolrd init" << endl;

  colmap.load(level_data->get_mapfile());
  cout << "Loaded Colmap" << endl;

  // load the foreground map
  if (level_data->map_type() == TILE)
    map = new PinguTileMap(level_data, resource);
  else
    map = new PinguBmpMap(level_data->get_fg(), resource);
  cout << "Loaded pingu map" << endl;

  // load the background map
  bg = new Background(level_data->get_bg(), resource);
  cout << "Loaded Background" << endl;

  allowed_pingus = level_data->get_pingus();
  result->set_total(allowed_pingus);

  fg_is_drawn = false;
  pg_is_drawn = false;
  bg_is_drawn = false;
  
  int pal;
  srand(time(NULL));

  // Creating Exit and Entrance
  vector<exit_data>     exit_d     = level_data->get_exit();
  vector<entrance_data> entrance_d = level_data->get_entrance();

  for(int i=0; i < exit_d.size(); ++i) 
    exits.push_back(new Exit(exit_d[i]));
  
  for(int i=0; i < entrance_d.size(); ++i) 
    entrance.push_back(new Entrance(entrance_d[i]));

  vector<trap_data>     trap_d     = level_data->get_traps();
  
  for(int i=0; i < trap_d.size(); ++i)
    traps.push_back(get_trap(trap_d[i]));

  // Creating the foreground and background hotspots
  vector<hotspot_t> hspot_fg = level_data->get_hotspot_fg();
  vector<hotspot_t> hspot_bg = level_data->get_hotspot_bg();

  for(int i=0; i < hspot_bg.size(); ++i)
    hotspot_bg.push_back(new Hotspot(hspot_bg[i], resource));

  for(int i=0; i < hspot_fg.size(); ++i)
    hotspot_fg.push_back(new Hotspot(hspot_fg[i], resource));
 
  // Setting Pingus at random positions
  pingu.resize(10);

  for(int i=0; i < pingu.size(); ++i) 
    pingu[i] = new Pingu(rand() % 800, rand() % 300);

  Pingu::set_colmap(&colmap);
  Pingu::set_map(map);

  cout << "Finished Word init" << endl;
}

vector<Pingu* >*
World::get_pingu_p()
{
  return &pingu;
}

int
World::get_width()
{
  return map->get_width();  
}

int
World::get_height()
{
  return map->get_height();
}

void
World::write_results()
{
  result->set_total(allowed_pingus);
  result->set_saved(5);
  result->set_killed(10);
}

/* EOF */
